@@ -1,4 +1,4 @@
-import pyglet, math
+import math
 
 
 def distance(point_1=(0, 0), point_2=(0, 0)):
